#container div.wrapper > h1 {
    display: none;
}

#container div.noSwitches {
    margin: 20px 0 0 0;
    text-align: center;
}

#container div.toolbar {
    border-bottom: solid 1px #e5e5e5;
    padding-bottom: 20px;
}

#container div.toolbar span.switchTest {
    float: right;
}

#container div.toolbar span.switchTest input {
    font-size: 13px;
    padding: 5px;
    width: 200px;
}

#container div.toolbar span.search {
    padding-top: 3px;
    float: right;
}

#container div.toolbar ul.sort {
    float: right;
    list-style: none;
    padding: 10px 20px 10px 0;
    margin: 0 20px 0 0;
    border-right: 1px solid #EAEAEA;
}

#container div.toolbar ul.sort li {
    display: inline-block;
}

#container div.toolbar ul.sort li a {
    color: #999;
    display: block;
}

#container div.toolbar ul.sort li + li {
    margin-left: 10px;
}

#container div.toolbar[data-sort=-label] ul.sort li.label a,
#container div.toolbar[data-sort=-date_created] ul.sort li.date_created a,
#container div.toolbar[data-sort=-date_modified] ul.sort li.date_modified a,
#container div.toolbar[data-sort=label] ul.sort li.label a,
#container div.toolbar[data-sort=date_created] ul.sort li.date_created a,
#container div.toolbar[data-sort=date_modified] ul.sort li.date_modified a,
#container div.toolbar[data-sort=""] ul.sort li.date_modified a {
    color: #000;
    font-weight: bold;
}

#container div.toolbar[data-sort=-label] ul.sort li.label a:after,
#container div.toolbar[data-sort=-date_created] ul.sort li.date_created a:after,
#container div.toolbar[data-sort=-date_modified] ul.sort li.date_modified a:after {
    content: " ▴";
}

#container div.toolbar[data-sort=label] ul.sort li.label a:after,
#container div.toolbar[data-sort=date_created] ul.sort li.date_created a:after,
#container div.toolbar[data-sort=date_modified] ul.sort li.date_modified a:after,
#container div.toolbar[data-sort=""] ul.sort li.date_modified a:after {
    content: " ▾";
}


#container div.toolbar span.search input {
    width: 200px;
    padding: 6px;
    font-size: 13px;
}

#container table.switches {
    width: 100%;
    border-collapse: collapse;
}

#container table.switches tr:hover {
    background: #f9f9f9;
    /*cursor: pointer;*/
}

#container table.switches tr.hidden {
    display: none;
}

#container table.switches tr.collapsed .inner {
    display: none;
}

#container table.switches tr {
    border: 3px solid lightgray;
    border-width: 0 1px;
}
#container table.switches tr:nth-child(even) {
    border-color: #C5C5C5;
    background-color: #fcfcfc;
}

#container table.switches td {
    vertical-align: top;
    border-bottom: solid 1px #efefef;
    padding: 20px 10px;
}


#container table.switches td.name h4 {
    font-size: 100%;
    font-weight: bold;
    margin: 0;
}
#container table.switches td.name h4 small {
    color: #999;
    font-weight: normal;
    margin-left: 8px;
    font-family: "Inconsolata", arial;
}

#container table.switches td.name h5 {
    color: #777;
    font-size: 50%;
    font-weight: normal;
    margin: 0.5em 0 0 0;
}

#container table.switches td.name p {
    color: #666;
    margin: 10px 0 0 0;
}

#container table.switches td.name p.addCondition {
    font-size: 80%;
    margin-top: 15px;
}

#container table.switches td.name div.conditionsForm {
    display: none;
    margin-top: 10px;
    line-height: 25px;
    min-height: 25px;
    border: solid 1px #dfdfdf;
    background-color: #f0f0f0;
    padding: 5px 10px;
    vertical-align: middle;
    -webkit-border-radius: 3px;
    -moz-border-radius: 3px;
    -o-border-radius: 3px;;
}
#container table.switches td.name div.conditionsForm.visible {
    display: inline-block;
}

#container table.switches td.name div.conditionsForm div.fields,
#container table.switches td.name div.conditionsForm div.fields form {
    display: inline;
    color: #999;
}

#container table.switches td.name div.conditionsForm div.fields label {
}

#container table.switches td.name div.conditionsForm div.fields input {
    font-size: 12px;
    margin: 0 5px;
}
#container table.switches td.name div.conditionsForm div.fields button {
    margin-left: 5px;
}

#container table.switches td.name div.conditionsForm div.fields .helptext {
    display: block;
    color: #666;
    font-size: 85%;
}

#container table.switches td.name div.condition {
    display: block;
}

#container table.switches td.name div.conditions {
    margin: 15px 0 0 15px;
}

#container table.switches td.name div.conditions div.group {
    font-size: 93%;
    margin-top: 5px;
    line-height: 18px;
    padding-left: 100px;
    position: relative;
}

#container table.switches td.name div.conditions div.group label {
    position: absolute;
    left: 0;
    width: 100px;
    color: #999;
}

#container table.switches td.name div.conditions div.group span.value {
    background: #f0f0f0;
    padding: 2px 4px;
    margin-right: 2px;
}

#container table.switches td.name div.conditions div.group span.value a.delete-condition {
    text-decoration: none;
}

#container table.switches td.name div.conditions div.group span.value a.delete-condition img {
    vertical-align: middle;
    margin-bottom: 2px;
}

#container table.switches td.status {
    width: 250px;
    text-align: center;
}
#container table.switches td.status p {
    color: #999;
    margin-top: 15px;
    font-size: 85%;
    text-transform: lowercase;
    margin-bottom: 0;
}

#container table.switches td.actions {
    text-align: right;
    width: 70px;
}

#container table.switches td.actions button.edit,
#container table.switches td.actions button.edit span {
    height: 18px;
    width: 18px;
}

#container table.switches td.actions button.edit span {
    display: block;
    background: transparent url("../img/edit.png") no-repeat left 35%;
}

#container table.switches td.actions button.delete,
#container table.switches td.actions button.delete span {
    height: 18px;
    width: 18px;
}

#container table.switches td.actions button.delete span {
    display: block;
    background: transparent url("../img/delete.png") no-repeat left 35%;
}

#container table.switches td.status button.button.toggled[data-status="1"] {
    border-color: #636363;
    filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#A8A8A8', endColorstr='#7E7E7E'); /* for IE */
    background: -webkit-gradient(linear, left top, left bottom, from(#A8A8A8), to(#7E7E7E)); /* for webkit browsers */
    background: -moz-linear-gradient(top, #7E7E7E, #A8A8A8); /* for firefox 3.6+ */        
}

#container table.empty {
    display: none;
}